@@ -1,6 +1,6 @@
 # The MIT License
 #
-# Copyright (c) 2009-2010 Andreas Stuehrk
+# Copyright (c) 2009-2011 Andreas Stuehrk
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -20,11 +20,28 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+from __future__ import with_statement
 
 import imp
 import os
 import sys
+import warnings
+try:
+    from warnings import catch_warnings
+except ImportError:
+    import contextlib
+    @contextlib.contextmanager
+    def catch_warnings():
+        """Stripped-down version of `warnings.catch_warnings()`
+        (available in Py >= 2.6)."""
+        filters = warnings.filters
+        warnings.filters = list(filters)
+        try:
+            yield
+        finally:
+            warnings.filters = filters
 
+py3 = sys.version_info[:2] >= (3, 0)
 
 # The cached list of all known modules
 modules = set()
@@ -93,7 +110,11 @@ def find_modules(path):
         # Perhaps a zip file
         return
 
-    for name in os.listdir(path):
+    try:
+        filenames = os.listdir(path)
+    except EnvironmentError:
+        filenames = []
+    for name in filenames:
         if not any(name.endswith(suffix[0]) for suffix in imp.get_suffixes()):
             # Possibly a package
             if '.' in name:
@@ -103,10 +124,19 @@ def find_modules(path):
             # which ends with a python extension, so work around.
             continue
         name = os.path.splitext(name)[0]
+        if py3 and name == "badsyntax_pep3120":
+            # Workaround for issue #166
+            continue
         try:
-            fo, pathname, _ = imp.find_module(name, [path])
+            with catch_warnings():
+                warnings.simplefilter("ignore", ImportWarning)
+                fo, pathname, _ = imp.find_module(name, [path])
         except (ImportError, SyntaxError):
             continue
+        except UnicodeEncodeError:
+            # Happens with Python 3 when there is a filename in some
+            # invalid encoding
+            continue
         else:
             if fo is not None:
                 fo.close()
@@ -129,6 +159,12 @@ def find_all_modules(path=None):
         if not p:
             p = os.curdir
         for module in find_modules(p):
+            if not py3 and not isinstance(module, unicode):
+                try:
+                    module = module.decode(sys.getfilesystemencoding())
+                except UnicodeDecodeError:
+                    # Not importable anyway, ignore it
+                    continue
             modules.add(module)
             yield
 
