@@ -1,6 +1,6 @@
 # The MIT License
 #
-# Copyright (c) 2009-2010 the bpython authors.
+# Copyright (c) 2009-2011 the bpython authors.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -100,6 +100,18 @@ class AttrCleaner(object):
             setattr(type_, '__getattr__', __getattr__)
         # /Dark magic
 
+class _Repr(object):
+    """
+    Helper for `fixlongargs()`: Returns the given value in `__repr__()`.
+    """
+
+    def __init__(self, value):
+        self.value = value
+
+    def __repr__(self):
+        return self.value
+
+    __str__ = __repr__
 
 def parsekeywordpairs(signature):
     tokens = PythonLexer().get_tokens(signature)
@@ -156,14 +168,16 @@ def fixlongargs(f, argspec):
     keys = argspec[0][-len(values):]
     try:
         src = inspect.getsourcelines(f)
-    except IOError:
+    except (IOError, IndexError):
+        # IndexError is raised in inspect.findsource(), can happen in
+        # some situations. See issue #94.
         return
     signature = ''.join(src[0])
     kwparsed = parsekeywordpairs(signature)
 
     for i, (key, value) in enumerate(zip(keys, values)):
-        if len(str(value)) != len(kwparsed[key]):
-            values[i] = kwparsed[key]
+        if len(repr(value)) != len(kwparsed[key]):
+            values[i] = _Repr(kwparsed[key])
 
     argspec[3] = values
 
