@@ -1,6 +1,6 @@
 # The MIT License
 #
-# Copyright (c) 2009-2010 the bpython authors.
+# Copyright (c) 2009-2011 the bpython authors.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -340,6 +340,8 @@ class Repl(object):
         self._C = {}
         self.prev_block_finished = 0
         self.interact = Interaction(self.config)
+        self.ps1 = '>>> '
+        self.ps2 = '... '
         # previous pastebin content to prevent duplicate pastes, filled on call
         # to repl.pastebin
         self.prev_pastebin_content = ''
@@ -473,11 +475,15 @@ class Repl(object):
                             stack[-1][1] += 1
                         except TypeError:
                             stack[-1][1] = ''
+                    elif value == ':' and stack[-1][2] == 'lambda':
+                        stack.pop()
                 elif (token is Token.Name or token in Token.Name.subtypes or
                       token is Token.Operator and value == '.'):
                     stack[-1][0] += value
                 elif token is Token.Operator and value == '=':
                     stack[-1][1] = stack[-1][0]
+                elif token is Token.Keyword and value == 'lambda':
+                    stack.append(['', 0, value])
                 else:
                     stack[-1][0] = ''
             while stack[-1][2] in '[{':
@@ -534,9 +540,15 @@ class Repl(object):
         if not self.get_args():
             self.argspec = None
         elif self.current_func is not None:
-            self.docstring = pydoc.getdoc(self.current_func)
-            if not self.docstring:
+            try:
+                self.docstring = pydoc.getdoc(self.current_func)
+            except IndexError:
                 self.docstring = None
+            else:
+                # pydoc.getdoc() returns an empty string if no
+                # docstring was found
+                if not self.docstring:
+                    self.docstring = None
 
         cw = self.cw()
         cs = self.current_string()
@@ -588,7 +600,7 @@ class Repl(object):
 
         if not e and self.argspec:
             matches.extend(name + '=' for name in self.argspec[1][0]
-                           if name.startswith(cw))
+                           if isinstance(name, basestring) and name.startswith(cw))
             if py3:
                 matches.extend(name + '=' for name in self.argspec[1][4]
                                if name.startswith(cw))
@@ -658,8 +670,10 @@ class Repl(object):
 
         def process():
             for line in s.split('\n'):
-                if line.startswith('>>>') or line.startswith('...'):
-                    yield line[4:]
+                if line.startswith(self.ps1):
+                    yield line[len(self.ps1):]
+                elif line.startswith(self.ps2):
+                    yield line[len(self.ps2):]
                 elif line.rstrip():
                     yield "# OUT: %s" % (line,)
         return "\n".join(process())
@@ -732,7 +746,21 @@ class Repl(object):
         self.buffer.append(s)
 
         if insert_into_history:
-            self.rl_history.append(s)
+            if self.config.hist_length:
+                histfilename = os.path.expanduser(self.config.hist_file)
+                oldhistory = self.rl_history.entries
+                self.rl_history.entries = []
+                if os.path.exists(histfilename):
+                    self.rl_history.load(histfilename, getpreferredencoding())
+                self.rl_history.append(s)
+                try:
+                    self.rl_history.save(histfilename, getpreferredencoding(), self.config.hist_length)
+                except EnvironmentError, err:
+                    self.interact.notify("Error occured while writing to file %s (%s) " % (histfilename, err.strerror))
+                    self.rl_history.entries = oldhistory
+                    self.rl_history.append(s)
+            else:
+                self.rl_history.append(s)
 
         more = self.interp.runsource('\n'.join(self.buffer))
 
