@@ -2,7 +2,7 @@
 #
 # The MIT License
 #
-# Copyright (c) 2010 Marien Zwart
+# Copyright (c) 2010-2011 Marien Zwart
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -44,13 +44,18 @@ from optparse import Option
 
 from pygments.token import Token
 
-from bpython import args as bpargs, repl
+from bpython import args as bpargs, repl, translations
 from bpython.formatter import theme_map
 from bpython.importcompletion import find_coroutine
+from bpython.translations import _
+
+from bpython.keys import urwid_key_dispatch as key_dispatch
 
 import urwid
 
 py3 = sys.version_info[0] == 3
+if not py3:
+    import inspect
 
 Parenthesis = Token.Punctuation.Parenthesis
 
@@ -74,6 +79,8 @@ COLORMAP = {
     'd': 'default',
     }
 
+# Add our keys to the urwid command_map
+
 
 try:
     from twisted.internet import protocol
@@ -212,12 +219,13 @@ class BPythonEdit(urwid.Edit):
 
     signals = ['edit-pos-changed']
 
-    def __init__(self, tab_length, *args, **kwargs):
+    def __init__(self, config, *args, **kwargs):
         self._bpy_text = ''
         self._bpy_attr = []
         self._bpy_selectable = True
         self._bpy_may_move_cursor = False
-        self.tab_length = tab_length
+        self.config = config
+        self.tab_length = config.tab_length 
         urwid.Edit.__init__(self, *args, **kwargs)
 
     def set_edit_pos(self, pos):
@@ -280,19 +288,30 @@ class BPythonEdit(urwid.Edit):
         return False
 
     def keypress(self, size, key):
+        if urwid.command_map[key] in ['cursor up', 'cursor down']:
+            # Do not handle up/down arrow, leave them for the repl.
+            return key
+
         self._bpy_may_move_cursor = True
         try:
-            # Do not handle up/down arrow, leave them for the repl.
-            if urwid.command_map[key] in ('cursor up', 'cursor down'):
-                return key
+            if urwid.command_map[key] == 'cursor max left':
+                self.edit_pos = 0
+            elif urwid.command_map[key] == 'cursor max right':
+                self.edit_pos = len(self.get_edit_text())
             elif key == 'backspace':
                 line = self.get_edit_text()
                 cpos = len(line) - self.edit_pos
                 if not (cpos or len(line) % self.tab_length or line.strip()):
                     self.set_edit_text(line[:-self.tab_length])
-                    return None
-            # TODO: Add in specific keypress fetching code here
-            return urwid.Edit.keypress(self, size, key)
+                else:
+                    return urwid.Edit.keypress(self, size, key)
+            elif key == 'pastebin':
+                # do pastebin
+                pass
+            else:
+                # TODO: Add in specific keypress fetching code here
+                return urwid.Edit.keypress(self, size, key)
+            return None
         finally:
             self._bpy_may_move_cursor = False
 
@@ -303,6 +322,14 @@ class BPythonEdit(urwid.Edit):
         finally:
             self._bpy_may_move_cursor = False
 
+class BPythonListBox(urwid.ListBox):
+    """Like `urwid.ListBox`, except that it does not eat up and
+    down keys.
+    """
+    def keypress(self, size, key):
+        if key not in ["up", "down"]:
+            return urwid.ListBox.keypress(self, size, key)
+        return key
 
 class Tooltip(urwid.BoxWidget):
 
@@ -391,7 +418,12 @@ class URWIDInteraction(repl.Interaction):
 
     def confirm(self, q):
         """Ask for yes or no and return boolean"""
-        return self.statusbar.prompt(q).lower().startswith('y')
+        try:
+            reply = self.statusbar.prompt(q)
+        except ValueError:
+            return False
+
+        return reply.lower() in (_('y'), _('yes'))
 
     def notify(self, s, n=10):
         return self.statusbar.message(s, n)
@@ -402,15 +434,14 @@ class URWIDRepl(repl.Repl):
     def __init__(self, event_loop, palette, interpreter, config):
         repl.Repl.__init__(self, interpreter, config)
 
-        self.listbox = urwid.ListBox(urwid.SimpleListWalker([]))
+        self.listbox = BPythonListBox(urwid.SimpleListWalker([]))
 
         # String is straight from bpython.cli
-        self.statusbar = Statusbar(
-            config,
-            " <%s> Rewind  <%s> Save  <%s> Pastebin  <%s> Pager  <%s> Show Source " %
-            (config.undo_key, config.save_key,
-             config.pastebin_key, config.last_output_key,
-             config.show_source_key))
+        self.statusbar = Statusbar(config,
+            _(" <%s> Rewind  <%s> Save  <%s> Pastebin "
+              " <%s> Pager  <%s> Show Source ") %
+              (config.undo_key, config.save_key, config.pastebin_key,
+               config.last_output_key, config.show_source_key))
 
         self.interact = URWIDInteraction(self.config, self.statusbar)
 
@@ -430,24 +461,34 @@ class URWIDRepl(repl.Repl):
         self.main_loop = urwid.MainLoop(
             self.frame, palette,
             event_loop=event_loop, unhandled_input=self.handle_input,
-            input_filter=input_filter)
+            input_filter=input_filter, handle_mouse=False)
 
         self.edits = []
         self.edit = None
+        self.current_output = None
         self._completion_update_suppressed = False
 
+        load_urwid_command_map(config)
+
     # Subclasses of Repl need to implement echo, current_line, cw
-    def echo(self, s):
-        s = s.rstrip('\n')
+    def echo(self, orig_s):
+        s = orig_s.rstrip('\n')
         if s:
-            text = urwid.Text(('output', s))
-            if self.edit is None:
-                self.listbox.body.append(text)
+            if self.current_output is None:
+                self.current_output = urwid.Text(('output', s))
+                if self.edit is None:
+                    self.listbox.body.append(self.current_output)
+                else:
+                    self.listbox.body.insert(-1, self.current_output)
+                    # The edit widget should be focused and *stay* focused.
+                    # XXX TODO: make sure the cursor stays in the same spot.
+                    self.listbox.set_focus(len(self.listbox.body) - 1)
             else:
-                self.listbox.body.insert(-1, text)
-                # The edit widget should be focused and *stay* focused.
-                # XXX TODO: make sure the cursor stays in the same spot.
-                self.listbox.set_focus(len(self.listbox.body) - 1)
+                # XXX this assumes this all has "output" markup applied.
+                self.current_output.set_text(
+                    ('output', self.current_output.text + s))
+        if orig_s.endswith('\n'):
+            self.current_output = None
         # TODO: maybe do the redraw after a short delay
         # (for performance)
         self.main_loop.draw_screen()
@@ -472,7 +513,7 @@ class URWIDRepl(repl.Repl):
         # Stolen from cli. TODO: clean up and split out.
         if (not text or
             (not text[-1].isalnum() and text[-1] not in ('.', '_'))):
-            return
+             return
 
         # Seek backwards in text for the first non-identifier char:
         for i, c in enumerate(reversed(text)):
@@ -516,7 +557,7 @@ class URWIDRepl(repl.Repl):
                 # does clever wrapping. I do not (yet).
                 for k, i in enumerate(args):
                     if defaults and k + 1 > len(args) - len(defaults):
-                        kw = str(defaults[k - (len(args) - len(defaults))])
+                        kw = repr(defaults[k - (len(args) - len(defaults))])
                     else:
                         kw = None
 
@@ -528,8 +569,14 @@ class URWIDRepl(repl.Repl):
                     if k == in_arg or i == in_arg:
                         color = 'bold ' + color
 
-                    markup.append((color, str(i)))
-                    if kw:
+                    if not py3:
+                        # See issue #138: We need to format tuple unpacking correctly
+                        # We use the undocumented function inspection.strseq() for
+                        # that. Fortunately, that madness is gone in Python 3.
+                        markup.append((color, inspect.strseq(i, str)))
+                    else:
+                        markup.append((color, str(i)))
+                    if kw is not None:
                         markup.extend([('punctuation', '='),
                                        ('token', kw)])
                     if k != len(args) - 1:
@@ -597,7 +644,12 @@ class URWIDRepl(repl.Repl):
 
     def ask_confirmation(self, q):
         """Ask for yes or no and return boolean"""
-        return self.statusbar.prompt(q).lower().startswith('y')
+        try:
+            reply = self.statusbar.prompt(q)
+        except ValueError:
+            return False
+
+        return reply.lower() in ('y', 'yes')
 
     def reevaluate(self):
         """Clear the buffer, redraw the screen and re-evaluate the history"""
@@ -685,8 +737,6 @@ class URWIDRepl(repl.Repl):
             signal.signal(signal.SIGINT, orig_handler)
 
     def start(self):
-        # Stolen from bpython.cli again
-        self.push('from bpython._internal import _help as help\n', False)
         self.prompt(False)
 
     def keyboard_interrupt(self):
@@ -694,17 +744,21 @@ class URWIDRepl(repl.Repl):
         self.echo('KeyboardInterrupt')
 
     def prompt(self, more):
+        # Clear current output here, or output resulting from the
+        # current prompt run will end up appended to the edit widget
+        # sitting above this prompt:
+        self.current_output = None
         # XXX is this the right place?
         self.rl_history.reset()
         # XXX what is s_hist?
         if not more:
-            self.edit = BPythonEdit(self.config.tab_length,
-                                    caption=('prompt', '>>> '))
-            self.stdout_hist += '>>> '
+            self.edit = BPythonEdit(self.config,
+                                    caption=('prompt', self.ps1))
+            self.stdout_hist += self.ps1
         else:
-            self.edit = BPythonEdit(self.config.tab_length,
-                                    caption=('prompt_more', '... '))
-            self.stdout_hist += '... '
+            self.edit = BPythonEdit(self.config,
+                                    caption=('prompt_more', self.ps2))
+            self.stdout_hist += self.ps2
 
         urwid.connect_signal(self.edit, 'change', self.on_input_change)
         urwid.connect_signal(self.edit, 'edit-pos-changed',
@@ -833,9 +887,7 @@ class URWIDRepl(repl.Repl):
             self._completion_update_suppressed = False
 
 def main(args=None, locals_=None, banner=None):
-    # Err, somewhat redundant. There is a call to this buried in urwid.util.
-    # That seems unfortunate though, so assume that's going away...
-    locale.setlocale(locale.LC_ALL, '')
+    translations.init()
 
     # TODO: maybe support displays other than raw_display?
     config, options, exec_args = bpargs.parse(args, (
@@ -967,11 +1019,6 @@ def main(args=None, locals_=None, banner=None):
                     continue
                 break
 
-            if config.hist_length:
-                histfilename = os.path.expanduser(config.hist_file)
-                myrepl.rl_history.save(histfilename,
-                                       locale.getpreferredencoding())
-
         finally:
             sys.stdin = orig_stdin
             sys.stderr = orig_stderr
@@ -1014,6 +1061,27 @@ def main(args=None, locals_=None, banner=None):
         sys.stdout.write(myrepl.getstdout())
     sys.stdout.flush()
 
-
+def load_urwid_command_map(config):
+    urwid.command_map[key_dispatch[config.up_one_line_key]] = 'cursor up'
+    urwid.command_map[key_dispatch[config.down_one_line_key]] = 'cursor down'
+    urwid.command_map[key_dispatch['C-a']] = 'cursor max left'
+    urwid.command_map[key_dispatch['C-e']] = 'cursor max right'
+"""
+            'clear_line': 'C-u',
+            'clear_screen': 'C-l',
+            'clear_word': 'C-w',
+            'cut_to_buffer': 'C-k',
+            'delete': 'C-d',
+            'down_one_line': 'C-n',
+            'exit': '',
+            'last_output': 'F9',
+            'pastebin': 'F8',
+            'save': 'C-s',
+            'show_source': 'F2',
+            'suspend': 'C-z',
+            'undo': 'C-r',
+            'up_one_line': 'C-p',
+            'yank_from_buffer': 'C-y'},
+"""
 if __name__ == '__main__':
     main()
